// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "ATTOAuthViewController.h"
#import "OAuthToken.h"
#import "ATTAppConfiguration.h"
#import <AFNetworking/AFHTTPClient.h>
#import <AFNetworking/AFJSONRequestOperation.h>
#import "ATTOAuthClient.h"

@interface ATTOAuthViewController ()

@end

@implementation ATTOAuthViewController{
    ATTAppConfiguration* _appConfig;
    OAuthToken* _token;
    NSString* _oauthPath;
    NSString* _tokenPath;
    AFHTTPClient* _httpClient;
    UIWebView* _webView;
}

#pragma mark - Public methods - Initial Configuration

-(void)setConfiguration:(ATTAppConfiguration *)appConfig{
    
    _appConfig= appConfig;
    
    // Initialize an HTTP client for JSON requests
    _httpClient= [[AFHTTPClient alloc] initWithBaseURL:_appConfig.baseURL];
    [_httpClient registerHTTPOperationClass:[AFJSONRequestOperation class]];
}

#pragma mark - UI Initialization & UIView Delegate

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self setupAppConfig];
    [self setupWebView];
}

- (void)setupWebView
{
    // create a WebView
    _webView= [[UIWebView alloc] init];
    // register the delegate to handle the redirection events
    _webView.delegate= self;
    
    // we're using autolayout, so don't translate autoresizing mask.
    _webView.translatesAutoresizingMaskIntoConstraints=NO;
    
    // add the webview to the current view
    [self.view addSubview:_webView];
    
    // position the webView
    [self.view addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|[_webView]|"
                                                                      options:0
                                                                      metrics:nil
                                                                        views:@{@"_webView": _webView}]];
    [self.view addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|[_webView]|"
                                                                      options:0
                                                                      metrics:nil
                                                                        views:@{@"_webView": _webView}]];
    
    // if no URL was specified, load AT&T's website
    if (!_appConfig.baseURL) {
        NSLog(@"Default URL: https://att.com");
        [_webView loadRequest:[NSURLRequest requestWithURL:
                              [NSURL URLWithString:@"https://att.com"]]];
        return;
    }
    
    // load the url given by the developer
    NSDictionary* params= @{@"client_id": _appConfig.key,
                            @"scope": _appConfig.scope,
                            @"redirect_uri": [_appConfig.redirectURL absoluteString]};
    NSMutableURLRequest* oauthRequest= [_httpClient requestWithMethod:@"GET"
                                                                 path:_oauthPath
                                                           parameters:params];
    [_webView loadRequest: oauthRequest];
    
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


#pragma mark - Helpers

-(void)setupAppConfig{
    _oauthPath= OAuthAuthorizationPath;
    _tokenPath= OAuthTokenPath;
}

/** Returns the query string of `url` as a dictionary.
 */
-(NSDictionary*)getDictionaryFromURL:(NSURL*)url{
    
    NSMutableDictionary* urlParams= [NSMutableDictionary dictionary];
    
    NSArray* urlComponents= [[url query
                              ] componentsSeparatedByString:@"&"];
    
    for (NSString* keyValuePair in urlComponents) {
        NSArray* pairComponents= [keyValuePair componentsSeparatedByString:@"="];
        if ([pairComponents count] == 2) {
            [urlParams setObject:pairComponents[1]
                          forKey:pairComponents[0]];
        }
    }
    return urlParams;
}

#pragma mark - UIWebViewDelegate

/** Here's where we handle all redirections inside the webView:
 
 When the user allows the application for the first time, this event will
 bee triggered and we have to extract the access code in order to get the
 Authorization Token.
 */
-(BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request
navigationType:(UIWebViewNavigationType)navigationType{
    
    NSString* urlString= [request.URL absoluteString];
    // if the user cancels the dialog, dismiss the view
    NSRange cancelRange= [ urlString rangeOfString:@"cancel"
                                           options:NSCaseInsensitiveSearch];
    if (NSNotFound != cancelRange.location) {
        [self.delegate didCancel];
        return NO;
    }

    // if the user taps the Open Messaging button, open the SMS app.
    NSRange openMessagingRange= [urlString rangeOfString:@"sms:"
                                                 options:NSCaseInsensitiveSearch];
    if (NSNotFound != openMessagingRange.location) {
        [[UIApplication sharedApplication] openURL: request.URL];
    }
    
    // if we already have a token, just continue the redirection
    if (_token) {
        return YES;
    }
    
    // if we don't have a token, look in the query string for the authorization code
    // get the query string as a dictionary
    NSDictionary* urlParams= [self getDictionaryFromURL:request.URL];
    
    // get the authorization code
    NSString* encodedAuthCode= (urlParams)[@"code"];
    
    // if there's no authorization code in the query string, we don't care
    // about this particular redirection
    if (!encodedAuthCode) {
        // continue redirection
        return YES;
    }
   
    NSString* authCode = [encodedAuthCode stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
    
    // stop loading
    [_webView stopLoading];
    
    // if we find the authorization code in the query string
    // use that code to get the OAuth token
    ATTOAuthClient* attOAuth= [[ATTOAuthClient alloc] initWithAppConfig:_appConfig];
    [attOAuth getTokenWithCode:authCode success:^(OAuthToken* token) {
        // send the received token to the delegate.
    
        [self.delegate didGetToken:token];
        
        // save a copy of the token
        _token= token;
    } failure:^(NSError* error){
        [self.delegate didFailToObtainToken:error];
    }];
      
    // ignore the redirection
    return NO;
}


-(void)webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error{
    
    // Ignore "Fame Load Interrupted" errors. Seen after auth returns a redirect with 'code' in the querystring.
    if (error.code == 102 && [error.domain isEqual:@"WebKitErrorDomain"]) return;

    NSLog(@"%@", error);
    [self.delegate didFailToObtainToken:error];
}

@end
