// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

/** Provides working examples of how to use the IAMManager to
    access all of the In-App Messaging API resources. See here for
    details: http://developer.att.com/apis/in-app-messaging/docs
 */

#import "MyCustomXCTestCase.h"
#import <AFNetworking.h>

#import "IAMManagerMock.h"

#import "IAMQueue.h"
#import "IAMMessagesListRequest.h"
#import "IAMMessageRequest.h"
#import "IAMDeltasUpdateRequest.h"
#import "IAMMessageIndexInfoRequest.h"
#import "IAMMessageContentRequest.h"
#import "IAMCreateMessageIndexRequest.h"
#import "IAMNotificationConnectionRequest.h"
#import "IAMSendMessageRequest.h"
#import "IAMUpdateMessageRequest.h"
#import "IAMUpdateMessagesRequest.h"
#import "IAMDeleteMessageRequest.h"
#import "IAMDeleteMessagesRequest.h"

#import "OAuthToken.h"
#import "ATTAppConfiguration.h"
#import "IAMMessage.h"
#import "IAMMessageIndexInfo.h"

#import "NSObject+Description.h"


/** These are our assumptions for testing:
    - You have joined the AT&T Developer program: 
        https://developer.att.com/developer/flow/apiPlaygroundFlow.do?execution=e1s1
    - You have created an App under your AT&T Developer program account
      - You have OAuth credentials for your Application. See:
      - You have enabled the In-App Messaging API's in your app. (MIM & IMMN scopes)
    - You have a basic understanding of the resources you want to use: 
        https://developer.att.com/apis/in-app-messaging/docs#resources
 */
NSString* const APIBaseURL=@"https://api.att.com";
NSString* const RedirectURL= @"http://localhost";
NSString* const AppKey= @"7rxkasvlp0u3vxcwrbqyxmrtydjj3gtu";
NSString* const AppSecret= @"3oy59mfgl2giszod4pdwe9vwjm7uy5jl";
NSString* const AppScope= @"MIM,IMMN";

@interface TestIAMMessagingManager : MyCustomXCTestCase
{
    // This is just a wrapper for the application parameters like,
    // the base URL,  app key and secret, etc.
    ATTAppConfiguration* _appConfig;
    // Temporary block-safe object to check the returned value of
    // methods that return arrays of objects.
    __block NSArray* expectedArrayValue;
    // Temporary block-safe object to check the returned value of
    // methods that return an instance of IAMMessage.
    __block IAMMessage* expectedMessage;
}

// This will manage all your IAM requests, it's basically an HTTP client
// for the In-App Messaging API resources.
@property IAMManager* messagingManager;

@end


@implementation TestIAMMessagingManager{
    OAuthToken* token;
}

@synthesize messagingManager;

- (void)setUp
{
    [super setUp];
    
    // initialize the app configuration
    _appConfig= [[ATTAppConfiguration alloc] init];
    _appConfig.key= AppKey;
    _appConfig.secret= AppSecret;
    _appConfig.scope= AppScope;
    _appConfig.baseURL= [NSURL URLWithString:APIBaseURL];
    _appConfig.redirectURL= [NSURL URLWithString:RedirectURL];
    
    // At this moment we don't assign a delegate, since it's mainly used
    // for notifying that we need to show the OAuth authorization page to
    // get an authorization code. You can find an example of how that would
    // work by reviewing the OAuthExample target.
    messagingManager= [[IAMManager alloc] initWithConfig:_appConfig andDelegate:nil];
    
    // Make sure you're using a valid OAuth Token, the tests will fail otherwise.
    // You can get one by using the procedure described in your app's profile
    // in the developer portal (developer.att.com)
    // Alternative 1: Follow the procedure described in the section 2. OAuth in the
    // In-App messaging documentation: http://developer.att.com/apis/in-app-messaging/docs
    token= [[OAuthToken alloc] init];

    token.accessToken= @"VAOzEJFZkAwYkeZW5bmmf7r6D2rUqdjm";
    token.expirationDate= [NSDate dateWithTimeIntervalSinceNow:157679999];
    token.type= @"bearer";
    token.refreshToken= @"UApg8IgZdA7yU2iYA4NiMtj3utdTvPLZ";
    
    [messagingManager setOauthToken:token];
}

- (void)tearDown
{
    // Put teardown code here; it will be run once, after the last test case.
    [super tearDown];
}

/** Allows the developer to create an index cache for the customer's AT&T
    Message inbox with prior consent.
    Expected result: an NSValue with the boolean result, YES==Success
 */
-(void)testCreateMessageIndex{
    
    IAMCreateMessageIndexRequest* indexRequest= [[IAMCreateMessageIndexRequest alloc] init];
    
    __block id expectedResult= nil;
    [messagingManager sendAsynchronous:indexRequest success:^(id response) {
        XCTAssertTrue([response boolValue],
                  @"Error: expected YES but received: %@", [response boolValue]? @"YES": @"NO");
        NSLog(@"Successfully created the message Index.");
        expectedResult= response;
    } failure:^(NSError *error) {
        XCTFail(@"%@", error.localizedDescription);
    }];
    
    [self waitForMe:5];
    
    XCTAssertTrue([expectedResult boolValue], @"Operation failed.");
}

/** Shortcut method to send SMS messges.
 Expected result: nothing. It will only tell you when it fails.
 */
-(void)testSendSMS{
    
    __block id expectedResult= nil;
    
    [messagingManager sendSMS:@"This is a test message"
                           to:@"tel:4258026381,pp729k@att.com"
                      failure:^(NSError* error){
                          NSLog(@"%@", error.localizedDescription);
                          expectedResult= error;
                      }];
    
    [self waitForMe:5];
    
    XCTAssertNil(expectedResult, @"There was an error: %@", expectedResult);
}

/** NOTES:
 - For SMS messages the preferred method is to use the `sendSMS:to:failure`
 shortcut method.
 
 If you need more control over what you send, keep reading...
 
 Send an SMS message to a phone number, e-mail or short-code.
 Expected result: an NSString with the identifier of the message sent.
 NOTES:
 - The returned identifier it NOT the same as the messageId returned
 by when you use `IAMMessageRequest`.
 - This is kept for forward compatibility and troubleshooting.
 */
- (void)testSendMessage{
    
    IAMSendMessageRequest* sendRequest= [[IAMSendMessageRequest alloc] init];
    sendRequest.addresses= @[@"tel:4258026381", @"pp729k@att.com"];
    sendRequest.text= @"This is an In-App Messaging SMS message!";
    
    __block id expectedId= nil;
    [messagingManager sendAsynchronous:sendRequest success:^(id messageId) {
        XCTAssertNotNil(messageId, @"Invalid message Id: %@", messageId);
        NSLog(@"%@", messageId);
        expectedId= messageId;
    } failure:^(NSError *error) {
        XCTFail(@"%@", error);
    }];
    
    [self waitForMe:5];
    
    XCTAssertNotNil(expectedId, @"Invalid MessageId");
}

/** This shows how to send an MMS message with attachements.
 Expected result: an NSString with the identifier of the sent message.
 */
-(void)testSendMessageWithMedia{
    
    IAMSendMessageRequest* sendRequest= [[IAMSendMessageRequest alloc] init];
    sendRequest.addresses= @[@"tel:4258026381", @"pp729k@att.com"];
    sendRequest.text= @"Hellou!";
    sendRequest.subject= @"Hi, this is an MMS message";
    sendRequest.isGroup= YES;
    
    // get the image from a file
    NSString* filePath= [[NSBundle bundleForClass:[self class]]
                         pathForResource:@"ATT-logo" ofType:@"jpg"];
    UIImage* image= [UIImage imageWithContentsOfFile:filePath];
    
    // ADD an image to the message
    [sendRequest addImage:image];
    
    NSString* someTextFileContent= @"Contens of some file.";
    
    // ADD text as an attachment to the message
    [sendRequest addText: someTextFileContent];
    
    
    // TODO: Attaching video files is Not implemented yet
    // [sendRequest addVideo:videoFile];
    // TODO: Attaching audio files is Not implemented yet
    // [sendRequest addAudio:audioFile];
    // TODO: Attaching Personal Information Manager files, though it should
    // work if we send them as text, I think.
    // [sendRequest addPIM:pimFile]
    
    __block id expectedId= nil;
    [messagingManager sendAsynchronous:sendRequest success:^(id messageId) {
        XCTAssertNotNil(messageId, @"Invalid message Id: %@", messageId);
        if (messageId) {
            NSLog(@"Succesfully sent message with Id: %@", messageId);
        }
        
        expectedId= messageId;
    } failure:^(NSError *error) {
        XCTFail(@"%@", error);
    }];
    
    [self waitForMe:15];
    
    XCTAssertNotNil(expectedId, @"Invalid message Id");
}


/** Short cut for getting all messages given the defaul values for limit and offset.
    Expected result: An NSArray of IAMMessage objects
 */
-(void)testGetAllMessages{
    
    expectedArrayValue= nil;
    
    [messagingManager getAllMessages:^(NSArray* messages){
        // on success we get an NSArray of IAMMessage objects
        XCTAssert((nil != messages) && (0 < [messages count]), @"There are no messages.");
        NSLog(@"The messages are: %@", [messages getPropertiesDictionary] );
        expectedArrayValue= messages;
        
    }failure:^(NSError* error){
        XCTFail(@"%@", error);
    }];
    
    [self waitForMe:15];
    
    XCTAssertNotNil(expectedArrayValue, @"No messages in result:");
    
}

/** Apply basic filter for limiting the number of messages returned.
    Expected result: An NSArray of at most `limit` IAMMessage objects
 */
-(void)testGetMessageListWithLimit{
    
    IAMMessagesListRequest* request= [[IAMMessagesListRequest alloc] init];
    
    // This is where we set the limit of returned messages
    request.limit= 20;
    
    expectedArrayValue= nil;
    [messagingManager sendAsynchronous:request success:^(id messageList) {
        // on success we get an NSArray of IAMMessage objects
        NSArray* messages= messageList;
        XCTAssert(messages.count <= request.limit,
                  @"Error: Count missmatch, expected at most: %d messages but "
                  @"received: %d", request.limit, messages.count);
        NSLog(@"Received: %d messages", messages.count);
        XCTAssert(nil != messageList, @"The response is nil.");
        NSLog(@"The messages are: %@", [messageList getPropertiesDictionary]);
        
        expectedArrayValue= messages;
    } failure:^(NSError *error) {
        XCTFail(@"%@", error);
    }];
    
    [self waitForMe:5];
    
    XCTAssertNotNil(expectedArrayValue, @"No messages in result:");
}

/** Filtering Messages by Id 
    Expected result: An NSArray with IAMMessage objects whose Id's are the ones
    specified by de `messageIDsFilter`
 */
-(void)testGetMessageListWithMessageIdsFilter{
    
    IAMMessagesListRequest* request= [[IAMMessagesListRequest alloc] init];
    // You just pass an array of message Id's and that's it.
    request.messageIDsFilter= @[@"S1"];
    
    expectedArrayValue= nil;
    [messagingManager sendAsynchronous:request success:^(id messageList) {
        
        NSArray* messages= messageList;
        XCTAssert(messages.count == request.messageIDsFilter.count,
                  @"Error: Count missmatch, expected: %d messages but "
                  @"received: %d", request.messageIDsFilter.count, messages.count);
        NSLog(@"Received: %d messages", messages.count);
        XCTAssert(nil != messageList, @"The response is nil.");
        NSLog(@"The messages are: %@", [messageList getPropertiesDictionary]);
        
        expectedArrayValue= messages;
    } failure:^(NSError *error) {
        XCTFail(@"%@", error);
    }];
    
    [self waitForMe:15];
    
    XCTAssertNotNil(expectedArrayValue, @"No messages in result:");
}

/** Get a single message.
    Expected result: an instance of IAMMessage representing the message with the 
    specified Id.
 */
- (void)testGetMessage
{

    IAMMessageRequest* request= [[IAMMessageRequest alloc] init];
    request.messageID= @"S52";
    
    expectedMessage= nil;
    [messagingManager sendAsynchronous:request success:^(id messageFound) {

        IAMMessage* message= messageFound;
        XCTAssertNotNil(message, @"ERROR: could not get a valid message: %@", message);
        
        NSLog(@"Message received: %@", [message description]);
        expectedMessage= message;
        
    } failure:^(NSError *error) {
        XCTFail(@"%@", [error localizedDescription]);
    }];
    
    [self waitForMe:5];

    XCTAssertNotNil(expectedMessage, @"No messages in result:");
}

/** Getting the contents of a text part in an MMS message
 Expected result: an NSString with the attached text.
 NOTE: You have to use a valid messageId and part number
 for this test to pass.
 */
-(void)testGetMessageContentText{
    
    IAMMessageContentRequest* contentRequest= [[IAMMessageContentRequest alloc] init];
    // an MMS message has an array of `parts` each part has a number and a type
    // that can be Audio, Image or Text.
    contentRequest.messageId= @"S1";
    contentRequest.partNumber= 2;
    
    __block id expectedContent= nil;
    [messagingManager sendAsynchronous:contentRequest success:^(id content) {
        XCTAssert(nil != content, @"No content for request");
        NSLog(@"Content is: %@", [content description]);
        
        expectedContent= content;
    } failure:^(NSError *error) {
        XCTFail(@"%@", error);
    }];
    
    [self waitForMe:10];
    
    XCTAssertNotNil(expectedContent, @"Invalid content");
}

/** Getting the contents of an Image part in an MMS message
 Expected result: an UIImage with the attached image.
 NOTE: You have to use a valid messageId and part number
 for this test to pass.
 */
-(void)testGetMessageContentImage{
    
    IAMMessageContentRequest* contentRequest= [[IAMMessageContentRequest alloc] init];
    // use a valid messageId for this test to pass
    contentRequest.messageId= @"S34";
    // Make sure you pass the right part number
    contentRequest.partNumber= 0;
    
    __block id expectedImage= nil;
    [messagingManager sendAsynchronous:contentRequest success:^(id content) {
        XCTAssert(nil != content, @"No content for request");
        // The content should be an UIImage
        XCTAssert([content isKindOfClass:[UIImage class]], @"The content is not an image");
        NSLog(@"Content is: %@", [content description]);
        
        expectedImage= content;
    } failure:^(NSError *error) {
        XCTFail(@"%@", error);
    }];
    
    [self waitForMe:10];
    XCTAssertNotNil(expectedImage, @"Invalid image");
}

/** Get the latest state of the mailbox given the current state
    Expected results: 
      - an NSArray of IAMDelta objects
      - updated mailbox state for this IAMManager
 */
- (void)testGetDelta
{
    
    // save the current mailbox state
    NSString* oldState= messagingManager.mailboxState;
    
    // prepare the request
    IAMDeltasUpdateRequest* deltaRequest= [[IAMDeltasUpdateRequest alloc] init];
    
    // NOTE: Remember to use a valid state.
    deltaRequest.state= @"1389310071539";
    
    __block id expectedDelta=nil;
    [messagingManager sendAsynchronous:deltaRequest success:^(id deltas) {
        
        // the deltas object should NOT be nil, but it can be empty
        XCTAssert(nil != deltas , @"Invalid deltas update: %@", deltas);
        NSLog(@"These are the deltas found: \n%@", [deltas getPropertiesDictionary]);
        
        expectedDelta= deltas;
        
    } failure:^(NSError *error) {
        XCTFail(@"Are you using a valid state?:%@, \n%@",deltaRequest.state, error);
    }];
    
    [self waitForMe:15];

    XCTAssertNotNil(expectedDelta, @"Invalid delta");
    
    // the mailbox state should NOT be nil and
    // is should be different than the emtpy string
    XCTAssert(nil != messagingManager.mailboxState
              && 0<[messagingManager.mailboxState length],
              @"The mailbox state; %@ is invalid.", messagingManager.mailboxState);
    
    // print the old and the current state. They may be the same if there
    // are not updates since the last time we checked.
    NSLog(@"Mailbox state changed from %@\n to: \n %@",
          oldState, messagingManager.mailboxState);

}

/** Get the current mailbox state (Message Index Info) 
    Expected result: an instance of IAMMessageIndexInfo
 */
- (void)testGetMessageIndexInfo
{
    
    IAMMessageIndexInfoRequest* request= [[IAMMessageIndexInfoRequest alloc] init];
    
    __block NSString* newMailboxState= nil;
    
    [messagingManager sendAsynchronous:request success:^(id messageIndexInfo) {
        
        IAMMessageIndexInfo* mii= (IAMMessageIndexInfo*)messageIndexInfo;
        
        XCTAssert(nil != mii, @"Invalid index info: %@",
                  mii);
        XCTAssert(ICSError != mii.indexCacheStatus, @"Invalid index state: %@",
                  [IAMMessageIndexInfo formatCacheStatus:mii.indexCacheStatus]);
        
        NSLog(@"MessageIndexInfo: %@", [messageIndexInfo getPropertiesDictionary]);
        
        // get the mailbox state
        newMailboxState= mii.mailboxState;
        
    } failure:^(NSError *error) {
        // failure code
        XCTFail(@"%@", error);
    }];
    
    [self waitForMe:15];
    
    XCTAssert([newMailboxState isEqualToString:messagingManager.mailboxState],
              @"Current Mailbox state: %@ does NOT match expected state: %@",
              newMailboxState, messagingManager.mailboxState);
    
}

/** Get the details about the credentials, endpoint, and resource information 
    to set up a notification connection.
    Expected result: and instance of IAMNotificationConnection
*/
- (void)testGetNotificationConnectionDetails
{
    IAMNotificationConnectionRequest* request= [[IAMNotificationConnectionRequest alloc] init];
    request.queues= @[[NSNumber numberWithInt:IAMTextQueue],
                      [NSNumber numberWithInt:IAMMMSQueue]];
    
    __block id expectedConnection= nil;
    [messagingManager sendAsynchronous:request success:^(id connection) {
        XCTAssert(nil != connection, @"Invalid connection.");
        NSLog(@"Connection Details: %@", [connection getPropertiesDictionary]);
        
        expectedConnection= connection;
    } failure:^(NSError *error) {
        XCTFail(@"%@", error);
    }];
    
    [self waitForMe:5];
    
    XCTAssertNotNil(expectedConnection, @"Invalid Connection");
    
}


/** You can favorite messages or mark them as Read/Unread by updating its flags 
    Expected result: the string `Success`
 */
-(void)testUpdateMessage{
    
    IAMUpdateMessageRequest* updateRequest= [[IAMUpdateMessageRequest alloc] init];
    updateRequest.messageId= @"S52";
    
    // update the flags
    updateRequest.isUnread= NO;
    updateRequest.isFavorite= NO;
    
    __block id expectedResult= nil;
    [messagingManager sendAsynchronous:updateRequest success:^(id result) {
        XCTAssertTrue([result boolValue],
                  @"Error: Could not update message. Result: %@", [result boolValue]? @"YES":@"NO");
        NSLog(@"%@", result);
        expectedResult= result;
    } failure:^(NSError *error) {
        XCTFail(@"%@", error);
    }];
    
    [self waitForMe:5];
    XCTAssertTrue([expectedResult boolValue], @"Operation failed.");
}

/** You can update the flags of a list of messages with a single call 
    Expected result: the string `Success`
 */
-(void)testUpdateMessages{

    // Create a list of messages with and set the flags we want to update
    NSMutableArray* messages= [NSMutableArray array];
    
    for (int i=0; i<2; ++i) {
        IAMMessage* message= [[IAMMessage alloc] init];
        // Remember to make sure the messages you want to update exists.
        // the hard-coded Id's made sense only for my testing phone.
        NSString* msgId= [NSString stringWithFormat:@"S%d", 50+i];
        message.messageId= msgId;
        
        // these are the updated flags
        message.unread= NO;
        message.favorite= NO;
        
        [messages addObject:message];
    }

    __block id expectedResult= nil;
    
    // Just pass the array of messages with the updated flags to the request
    IAMUpdateMessagesRequest* request= [[IAMUpdateMessagesRequest alloc]
                                        initWithMessages:messages];
    
    [messagingManager sendAsynchronous:request success:^(id result) {
        XCTAssertTrue([result boolValue],
                      @"The operation failed with result: %@", [result boolValue]?@"YES":@"NO");
        NSLog(@"Request result: %@", result);
        
        expectedResult= result;
    } failure:^(NSError *error) {
        XCTFail(@"%@" ,error);
    }];
    
    [self waitForMe:5];
    XCTAssertTrue([expectedResult boolValue], @"Operation failed.");
}

/** You can delete a message using it's Id.
    Expected result: the string `Success`
 */
-(void)testDeleteMessage{

    IAMDeleteMessageRequest* delRequest= [[IAMDeleteMessageRequest alloc] init];
    delRequest.messageId= @"S50";
    
    __block id expectedResult= nil;
    [messagingManager sendAsynchronous:delRequest success:^(id result) {
        XCTAssertTrue([result boolValue],
                      @"The operation failed with result: %@", [result boolValue]?@"YES":@"NO");
        NSLog(@"Request result: %@", result);
        expectedResult= result;
    } failure:^(NSError *error) {
        XCTFail(@"%@" ,error);
    }];
    
    [self waitForMe:5];
    XCTAssertTrue([expectedResult boolValue], @"Operation failed.");
}

/** You can delete multiple messages with a single call. 
    Expected results: the string `Success`
 */
-(void)testDeleteMessages{
    
    // TODO: You could call getAllMessages and delete the last 2 messages
    // to avoid having to update the Id's manually.
    // create the message list for this test
    NSMutableArray* messages= [NSMutableArray array];
    for (int i=0; i<2; ++i) {
        IAMMessage* message= [[IAMMessage alloc] init];
        // remember to use valid Id's
        NSString* msgId= [NSString stringWithFormat:@"I%d", 51+i];
        message.messageId= msgId;
        
        [messages addObject:message];
    }
    
    // Just pass the list of messages to the request.
    IAMDeleteMessagesRequest* delRequest= [[IAMDeleteMessagesRequest
                                            alloc] initWithMessages:messages];
    
    __block id expectedResult= nil;
    [messagingManager sendAsynchronous:delRequest success:^(id result) {
        XCTAssertTrue([result boolValue],
                      @"The operation failed with result: %@", [result boolValue]?@"YES":@"NO");
        NSLog(@"Request result: %@", result);
        expectedResult= result;
    } failure:^(NSError *error) {
        XCTFail(@"%@" ,error);
    }];
    
    [self waitForMe:5];
    
    XCTAssertTrue([expectedResult boolValue], @"Operation failed.");
}

@end
